''' A User Model Service Provider '''
from masonite.provider import ServiceProvider
import os
from config.dashboard import MODELS
from .links import ExportLink, ModelLink, Home, Logout
from .Link import BaseLink, UserLink

package_directory = os.path.dirname(os.path.realpath(__file__))

class DashboardProvider(ServiceProvider):

    wsgi = False 

    def register(self):
       
       # Register static files
        self.app.make('Storage').STATICFILES.update({
            os.path.join(package_directory, 'static'): '_dashboard/' 
        })

        # Register Models
        self.app.bind('DashboardModels', MODELS)

        # Register Links
        self.app.bind('HomeLink', Home)

    def boot(self, Storage, ViewClass):
        ViewClass.add_environment('dashboard/templates')
        ViewClass.composer(['/dashboard*', 'dashboard*'], {'nav_links': self.app.collect(BaseLink), 'user_links': self.app.collect(UserLink)})

''' A UserManagementProvider Service Provider '''
from dashboard.links import UserManagementLink, SwapUserLink

class UserManagementProvider(ServiceProvider):

    wsgi = False

    def register(self):
        self.app.bind('UserManagementLink', UserManagementLink)
        self.app.bind('SwapLink', SwapUserLink)

    def boot(self, Request):
        Request.extend(RealUser)

class RealUser:

    def real_user(self):
        if self.get_cookie('_real_token'):
            return User.where('remember_token', self.get_cookie('_real_token')).first()
        
        return None


""" User Management """
from masonite.facades.Auth import Auth
from masonite.helpers import password
from masonite.request import Request
from masonite.view import View

from config.auth import AUTH
from config.database import DB


class UserManagementController:
    """ User Management Controller """

    __auth__ = AUTH['model']
    template_prefix = '/dashboard/templates/dashboard'

    def show(self, view: View, request: Request):
        if request.input('search'):
            users = self.__auth__.all().filter(lambda user: self.search(request.input('search'), user))
        else:
            users = self.__auth__.all()

        return view.render('{0}/management/management'.format(self.template_prefix), {'users': users})

    def search(self, search, item):
        if search.lower() in item.name.lower():
            return True

        if search.lower() in item.email.lower():
            return True


    def login_as_user(self, request: Request):
        if not request.get_cookie('_real_token'):
            request.cookie('_real_token', request.get_cookie('token'))

        Auth(request).login_by_id(request.input('user'))
        return request.redirect('/dashboard')


    def swap_back_to_user(self, request: Request):
        if request.get_cookie('_real_token'):
            request.cookie('token', request.get_cookie('_real_token'))
            request.delete_cookie('_real_token')

        return request.redirect('/dashboard')


    def delete_user(self, request: Request):
        self.__auth__.find(request.input('user')).delete()

        return request.redirect('/dashboard/management')


    def show_user(self, view: View):
        conn = DB.get_schema_manager().list_table_columns('users')
        return view.render('{0}/management/add_user'.format(self.template_prefix), {'schema': conn})


    def create(self, request: Request):
        del request.request_variables['csrf_token']

        # if POST /api/users
        proxy = self.__auth__()

        for field, value in request.all().items():
            if value:
                print(field)
                if field == 'password':

                    setattr(proxy, field, password(request.input('password')))
                else:
                    setattr(proxy, field, value)

        proxy.save()

        return request.redirect('/dashboard/management')



''' Welcome The User To Masonite '''
from masonite.view import View
from masonite.facades.Auth import Auth


class AuthenticationController:

    def show(self, view: View):
        """ Show Login Page """
        return view.render('/dashboard/templates/dashboard/login')

    def authenticate(self, Request):
        user = Auth(Request).login(Request.input('username'), Request.input('password'))
        if user and user.is_admin:
           return Request.redirect('/dashboard')
        elif not user:
            Request.session.flash('danger', 'Username or password is incorrect')
        else:
            Request.session.flash('danger', 'User does not have admin priviledges')
        
        Request.redirect('/dashboard/login')   

    def logout(self, Request):
        Auth(Request).logout()
        return Request.redirect('/dashboard/login')


from .Link import BaseLink, UserLink

class ExportLink(BaseLink):
    display = 'Export'
    url = 'http://google.com'

class ModelLink(BaseLink):
    display = 'Models'
    url = '/dashboard'

class ExportLink(BaseLink):
    display = 'Export'
    url = 'http://google.com'

class ModelLink(BaseLink):
    display = 'Models'
    url = '/dashboard'

class Home(BaseLink):
    display = 'Home'
    url = '/dashboard'

class Logout(UserLink):
    display = 'Logout'
    url = '/logout'

from dashboard.Link import BaseLink, UserLink
from masonite.request import Request

class UserManagementLink(BaseLink):
    display = 'User Management'
    url = '/dashboard/management'
    
    def visible(self, request: Request):
        if request.get_cookie('_real_token'):
            return True
    
    def show(self):
        return self.display

class SwapUserLink(UserLink):
    display = 'Swap Back To User'
    url = '/dashboard/management/swap'

    def visible(self, request: Request):
        if request.get_cookie('_real_token'):
            return True
    
    def show(self):
        return self.display


    
from masonite.helpers.routes import get, post
from dashboard.controllers.UserManagementController import UserManagementController
def routes(prefix='/dashboard'):
    return [
        get('/dashboard', '/dashboard.controllers.DashboardController@show'),
        get('/dashboard/login', '/dashboard.controllers.AuthenticationController@show'),
        get('/dashboard/logout', '/dashboard.controllers.AuthenticationController@logout'),
        post('/dashboard/login', '/dashboard.controllers.AuthenticationController@authenticate'),
    ]

def management_routes():
    return [
        get('/dashboard/management', UserManagementController.show),
        post('/dashboard/management/login_as_user', UserManagementController.login_as_user),
        get('/dashboard/management/swap', UserManagementController.swap_back_to_user),
        post('/dashboard/management/delete', UserManagementController.delete_user),
        get('/dashboard/management/show', UserManagementController.show_user),
        post('/dashboard/management/create', UserManagementController.create),
    ]



class BaseLink:
    display = ''
    url = ''

    def visible(self):
        return True

    def show(self):
        return self.display

class UserLink:
    display = ''
    url = ''

    def visible(self):
        return True

    def show(self):
        return self.display


''' Welcome The User To Masonite '''
from app.User import User
from config.database import DB
from masonite.request import Request

class DashboardController:

    def show(self, request: Request):
        if request.user() and request.user().is_admin:
            return view('/dashboard/templates/dashboard/index')
        
        if request.user():
            request.session.flash('danger', 'You are not an admin')
        return request.redirect('/dashboard/login')
    
    # def single(self):
    #     model = request().app().make('DashboardModels')[request().param('model')]
    #     if model.__table__:
    #         table_name = model.__table__
    #     else:
    #         table_name = model.__name__ + 's'

    #     conn = DB.get_schema_manager().list_table_columns(table_name.lower())
        
    #     model = model.find(request().param('id'))
    #     return view('/dashboard/templates/single', {'model': model, 'getattr': getattr, 'model_schema': conn.items()})

    # def show_all(self):
    #     model = request().app().make('DashboardModels')[request().param('model')]
    #     models = model.all()
    #     return view('/dashboard/templates/show', {'modelclass': model, 'models': models})

    # def update(self):
    #     model = request().app().make('DashboardModels')[request().param('model')]()

    #     model = model.find(request().param('id'))
        
    #     for key in request().all():
    #         if hasattr(model, key):
    #             setattr(model, key, request().input(key))

    #     model.save()
    #     return request().redirect('/dashboard/{}/{}'.format(model.__class__.__name__.lower(), request().param('id'))) \
    #         .session.flash('message', 'Updated Successfully')


from setuptools import setup

setup(
    name='masonite-dashboard',
    packages=[
        'dashboard',
        'dashboard.controllers',
        'dashboard',
    ],
    version='0.1.1',
    install_requires=[
        
    ],
    description='Masonite Dashboard',
    author='Joseph Mancuso',
    author_email='idmann509@gmail.com',
    url='https://github.com/MasoniteFramework/masonite',
    keywords=['python web framework', 'python3'],
    classifiers=[],
    include_package_data=True,
)
